#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
inkex.py
A helper module for creating Inkscape extensions

Copyright (C) 2005,2010 Aaron Spike <aaron@ekips.org> and contributors

Contributors:
  Aurélio A. Heckert <aurium(a)gmail.com>
  Bulia Byak <buliabyak@users.sf.net>
  Nicolas Dufour, nicoduf@yahoo.fr
  Peter J. R. Moulder <pjrm@users.sourceforge.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import argparse
import copy
import gettext
import os
import random
import re
import sys

from math import *

# a dictionary of all of the xmlns prefixes in a standard inkscape doc
NSS = {
u'sodipodi' :u'http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd',
u'cc'       :u'http://creativecommons.org/ns#',
u'ccOLD'    :u'http://web.resource.org/cc/',
u'svg'      :u'http://www.w3.org/2000/svg',
u'dc'       :u'http://purl.org/dc/elements/1.1/',
u'rdf'      :u'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
u'inkscape' :u'http://www.inkscape.org/namespaces/inkscape',
u'xlink'    :u'http://www.w3.org/1999/xlink',
u'xml'      :u'http://www.w3.org/XML/1998/namespace'
}


def localize():
    domain = 'inkscape'
    if sys.platform.startswith('win'):
        import locale
        current_locale, encoding = locale.getdefaultlocale()
        os.environ['LANG'] = current_locale
        try:
            localdir = os.environ['INKSCAPE_LOCALEDIR']
            trans = gettext.translation(domain, localdir, [current_locale], fallback=True)
        except KeyError:
            trans = gettext.translation(domain, fallback=True)
    elif sys.platform.startswith('darwin'):
        try:
            localdir = os.environ['INKSCAPE_LOCALEDIR']
            trans = gettext.translation(domain, localdir, fallback=True)
        except KeyError:
            try:
                localdir = os.environ['PACKAGE_LOCALE_DIR']
                trans = gettext.translation(domain, localdir, fallback=True)
            except KeyError:
                trans = gettext.translation(domain, fallback=True)
    else:
        try:
            localdir = os.environ['PACKAGE_LOCALE_DIR']
            trans = gettext.translation(domain, localdir, fallback=True)
        except KeyError:
            trans = gettext.translation(domain, fallback=True)
    #sys.stderr.write(str(localdir) + "\n")
    trans.install()


def debug(what):
    sys.stderr.write(str(what) + "\n")
    return what


def errormsg(msg):
    """Intended for end-user-visible error messages.
    
       (Currently just writes to stderr with an appended newline, but could do
       something better in future: e.g. could add markup to distinguish error
       messages from status messages or debugging output.)
      
       Note that this should always be combined with translation:

         import inkex
         ...
         inkex.errormsg(_("This extension requires two selected paths."))
    """
    if sys.version_info.major == 2:
        if isinstance(msg, unicode):
            sys.stderr.write(msg.encode("utf-8") + "\n")
        else:
            sys.stderr.write((unicode(msg, "utf-8", errors='replace') + "\n").encode("utf-8"))
    else:
        sys.stderr.write(msg + "\n")

def are_near_relative(a, b, eps):
    return (a-b <= a*eps) and (a-b >= -a*eps)


# third party library
try:
    from lxml import etree
except ImportError as e:
    localize()
    errormsg(_("The fantastic lxml wrapper for libxml2 is required by inkex.py and therefore this extension."
               "Please download and install the latest version from http://cheeseshop.python.org/pypi/lxml/, "
               "or install it through your package manager by a command like: sudo apt-get install "
               "python-lxml\n\nTechnical details:\n%s" % (e, )))
    sys.exit()


def boolean_option(value):
    ''' use for boolean options added to an argparse.ArgumentParser, like:

        parser.add_argument("--flag", type=boolean_option)
    '''
    if str(value).upper() == "TRUE":
        return True
    elif str(value).upper() == "FALSE":
        return False
    raise argparse.ArgumentError("Value of argument must be TRUE or FALSE")

def addNS(tag, ns=None):
    val = tag
    if ns is not None and len(ns) > 0 and ns in NSS and len(tag) > 0 and tag[0] != '{':
        val = "{%s}%s" % (NSS[ns], tag)
    return val

class Effect:
    """A class for creating Inkscape SVG Effects"""

    def __init__(self, *args, **kwargs):
        self.document = None
        self.original_document = None
        self.ctx = None
        self.selected = {}
        self.doc_ids = {}
        self.options = None

        self.arg_parser = argparse.ArgumentParser(
                parents=kwargs.get('common_options', []),
                usage="usage: %(prog)s [options] input_file")
        self.arg_parser.add_argument("--id",
                        action="append", type=str, dest="ids", default=[],
                        help="id attribute of object to manipulate")
        self.arg_parser.add_argument("--selected-nodes",
                        action="append", type=str, dest="selected_nodes", default=[],
                        help="id:subpath:position of selected nodes, if any")
        # TODO write a parser for this

    def effect(self):
        """Apply some effects on the document. Extensions subclassing Effect
        must override this function and define the transformations
        in it."""
        pass

    def getoptions(self,args=sys.argv[1:]):
        """Collect command line arguments"""
        self.options = self.arg_parser.parse_args(args)

    def parse(self, filename=None):
        """Parse document in specified file or on stdin"""

        # First try to open the file from the function argument
        if filename is not None:
            try:
                if sys.version_info < (3,):
                    stream = open(filename, 'r')
                else:
                    stream = open(filename, 'r', encoding="utf8")
            except IOError:
                errormsg(_("Unable to open specified file: %s") % filename)
                sys.exit()

        # If it wasn't specified, try to open the file specified as
        # an object member
        elif self.svg_file is not None:
            try:
                if sys.version_info < (3,):
                    stream = open(self.svg_file, 'r')
                else:
                    stream = open(self.svg_file, 'r', encoding="utf8")
            except IOError:
                errormsg(_("Unable to open object member file: %s") % self.svg_file)
                sys.exit()

        # Finally, if the filename was not specified anywhere, use
        # standard input stream
        else:
            stream = sys.stdin

        p = etree.XMLParser(huge_tree=True)
        self.document = etree.parse(stream, parser=p)
        self.original_document = copy.deepcopy(self.document)
        stream.close()

    # defines view_center in terms of document units
    def getposinlayer(self):
        #defaults
        self.current_layer = self.document.getroot()
        self.view_center = (0.0, 0.0)

        layerattr = self.document.xpath('//sodipodi:namedview/@inkscape:current-layer', namespaces=NSS)
        if layerattr:
            layername = layerattr[0]
            layer = self.document.xpath('//svg:g[@id="%s"]' % layername, namespaces=NSS)
            if layer:
                self.current_layer = layer[0]

        xattr = self.document.xpath('//sodipodi:namedview/@inkscape:cx', namespaces=NSS)
        yattr = self.document.xpath('//sodipodi:namedview/@inkscape:cy', namespaces=NSS)
        if xattr and yattr:
            x = self.unittouu(xattr[0] + 'px')
            y = self.unittouu(yattr[0] + 'px')
            doc_height = self.unittouu(self.getDocumentHeight())
            if x and y:
                self.view_center = (float(x), doc_height - float(y))
                # FIXME: y-coordinate flip, eliminate it when it's gone in Inkscape

    def getselected(self):
        """Collect selected nodes"""
        for i in self.options.ids:
            path = '//*[@id="%s"]' % i
            for node in self.document.xpath(path, namespaces=NSS):
                self.selected[i] = node

    def getElementById(self, id):
        path = '//*[@id="%s"]' % id
        el_list = self.document.xpath(path, namespaces=NSS)
        if el_list:
            return el_list[0]
        else:
            return None

    def getParentNode(self, node):
        for parent in self.document.getiterator():
            if node in parent.getchildren():
                return parent

    def getdocids(self):
        docIdNodes = self.document.xpath('//@id', namespaces=NSS)
        for m in docIdNodes:
            self.doc_ids[m] = 1

    def getNamedView(self):
        return self.document.xpath('//sodipodi:namedview', namespaces=NSS)[0]

    def createGuide(self, posX, posY, angle):
        atts = {
          'position': str(posX)+','+str(posY),
          'orientation': str(sin(radians(angle)))+','+str(-cos(radians(angle)))
          }
        guide = etree.SubElement(
                  self.getNamedView(),
                  addNS('guide','sodipodi'), atts)
        return guide

    def output(self):
        """Serialize document into XML on stdout"""
        original = etree.tostring(self.original_document)        
        result = etree.tostring(self.document)        
        if original != result:
            sys.stdout.write(result.decode("utf-8"))

    def affect(self, args=sys.argv[1:], output=True):
        """Affect an SVG document with a callback effect"""
        self.svg_file = args[-1]
        localize()
        self.getoptions(args[:-1]) # not the last term, because that's now in self.svg_file
        self.parse()
        self.getposinlayer()
        self.getselected()
        self.getdocids()
        self.effect()
        if output:
            self.output()

    def uniqueId(self, old_id, make_new_id=True):
        new_id = old_id
        if make_new_id:
            while new_id in self.doc_ids:
                new_id += random.choice('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ')
            self.doc_ids[new_id] = 1
        return new_id

    def xpathSingle(self, path):
        try:
            retval = self.document.xpath(path, namespaces=NSS)[0]
        except:
            errormsg(_("No matching node for expression: %s") % path)
            retval = None
        return retval

    # a dictionary of unit to user unit conversion factors
    __uuconv = {'in': 96.0, 'pt': 1.33333333333, 'px': 1.0, 'mm': 3.77952755913, 'cm': 37.7952755913,
                'm': 3779.52755913, 'km': 3779527.55913, 'pc': 16.0, 'yd': 3456.0, 'ft': 1152.0}

    # Fault tolerance for lazily defined SVG
    def getDocumentWidth(self):
        width = self.document.getroot().get('width')
        if width:
            return width
        else:
            viewbox = self.document.getroot().get('viewBox')
            if viewbox:
                return viewbox.split()[2]
            else:
                return '0'

    # Fault tolerance for lazily defined SVG
    def getDocumentHeight(self):
        """Returns a string corresponding to the height of the document, as
        defined in the SVG file. If it is not defined, returns the height
        as defined by the viewBox attribute. If viewBox is not defined,
        returns the string '0'."""
        height = self.document.getroot().get('height')
        if height:
            return height
        else:
            viewbox = self.document.getroot().get('viewBox')
            if viewbox:
                return viewbox.split()[3]
            else:
                return '0'

    def getDocumentUnit(self):
        """Returns the unit used for in the SVG document.
        In the case the SVG document lacks an attribute that explicitly
        defines what units are used for SVG coordinates, it tries to calculate
        the unit from the SVG width and viewBox attributes.
        Defaults to 'px' units."""
        svgunit = 'px'  # default to pixels

        svgwidth = self.getDocumentWidth()
        viewboxstr = self.document.getroot().get('viewBox')
        if viewboxstr:
            unitmatch = re.compile('(%s)$' % '|'.join(self.__uuconv.keys()))
            param = re.compile(r'(([-+]?[0-9]+(\.[0-9]*)?|[-+]?\.[0-9]+)([eE][-+]?[0-9]+)?)')

            p = param.match(svgwidth)
            u = unitmatch.search(svgwidth)

            width = 100  # default
            viewboxwidth = 100  # default
            svgwidthunit = 'px'  # default assume 'px' unit
            if p:
                width = float(p.string[p.start():p.end()])
            else:
                errormsg(_("SVG Width not set correctly! Assuming width = 100"))
            if u:
                svgwidthunit = u.string[u.start():u.end()]

            viewboxnumbers = []
            for t in viewboxstr.split():
                try:
                    viewboxnumbers.append(float(t))
                except ValueError:
                    pass
            if len(viewboxnumbers) == 4:  # check for correct number of numbers
                viewboxwidth = viewboxnumbers[2]

            svgunitfactor = self.__uuconv[svgwidthunit] * width / viewboxwidth

            # try to find the svgunitfactor in the list of units known. If we don't find something, ...
            eps = 0.01  # allow 1% error in factor
            for key in self.__uuconv:
                if are_near_relative(self.__uuconv[key], svgunitfactor, eps):
                    # found match!
                    svgunit = key

        return svgunit

    def unittouu(self, string):
        """Returns userunits given a string representation of units in another system"""
        unit = re.compile('(%s)$' % '|'.join(self.__uuconv.keys()))
        param = re.compile(r'(([-+]?[0-9]+(\.[0-9]*)?|[-+]?\.[0-9]+)([eE][-+]?[0-9]+)?)')

        p = param.match(string)
        u = unit.search(string)    
        if p:
            retval = float(p.string[p.start():p.end()])
        else:
            retval = 0.0
        if u:
            try:
                return retval * (self.__uuconv[u.string[u.start():u.end()]] / self.__uuconv[self.getDocumentUnit()])
            except KeyError:
                pass
        else:  # default assume 'px' unit
            return retval / self.__uuconv[self.getDocumentUnit()]

        return retval

    def uutounit(self, val, unit):
        return val / (self.__uuconv[unit] / self.__uuconv[self.getDocumentUnit()])

    def addDocumentUnit(self, value):
        """Add document unit when no unit is specified in the string """
        try:
            float(value)
            return value + self.getDocumentUnit()
        except ValueError:
            return value

# vim: expandtab shiftwidth=4 tabstop=8 softtabstop=4 fileencoding=utf-8 textwidth=99



#!/usr/bin/env python
"""
cubicsuperpath.py

Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
from . import simplepath
from math import *

def matprod(mlist):
    prod=mlist[0]
    for m in mlist[1:]:
        a00=prod[0][0]*m[0][0]+prod[0][1]*m[1][0]
        a01=prod[0][0]*m[0][1]+prod[0][1]*m[1][1]
        a10=prod[1][0]*m[0][0]+prod[1][1]*m[1][0]
        a11=prod[1][0]*m[0][1]+prod[1][1]*m[1][1]
        prod=[[a00,a01],[a10,a11]]
    return prod
def rotmat(teta):
    return [[cos(teta),-sin(teta)],[sin(teta),cos(teta)]]
def applymat(mat, pt):
    x=mat[0][0]*pt[0]+mat[0][1]*pt[1]
    y=mat[1][0]*pt[0]+mat[1][1]*pt[1]
    pt[0]=x
    pt[1]=y
def norm(pt):
    return sqrt(pt[0]*pt[0]+pt[1]*pt[1])

def ArcToPath(p1,params):
    A=p1[:]
    rx,ry,teta,longflag,sweepflag,x2,y2=params[:]
    teta = teta*pi/180.0
    B=[x2,y2]
    if rx==0 or ry==0 or A==B:
        return([[A[:],A[:],A[:]],[B[:],B[:],B[:]]])
    mat=matprod((rotmat(teta),[[1/rx,0],[0,1/ry]],rotmat(-teta)))
    applymat(mat, A)
    applymat(mat, B)
    k=[-(B[1]-A[1]),B[0]-A[0]]
    d=k[0]*k[0]+k[1]*k[1]
    k[0]/=sqrt(d)
    k[1]/=sqrt(d)
    d=sqrt(max(0,1-d/4))
    if longflag==sweepflag:
        d*=-1
    O=[(B[0]+A[0])/2+d*k[0],(B[1]+A[1])/2+d*k[1]]
    OA=[A[0]-O[0],A[1]-O[1]]
    OB=[B[0]-O[0],B[1]-O[1]]
    start=acos(OA[0]/norm(OA))
    if OA[1]<0:
        start*=-1
    end=acos(OB[0]/norm(OB))
    if OB[1]<0:
        end*=-1

    if sweepflag and start>end:
        end +=2*pi
    if (not sweepflag) and start<end:
        end -=2*pi

    NbSectors=int(abs(start-end)*2/pi)+1
    dTeta=(end-start)/NbSectors
    #v=dTeta*2/pi*0.552
    #v=dTeta*2/pi*4*(sqrt(2)-1)/3
    v = 4*tan(dTeta/4)/3
    #if not sweepflag:
    #    v*=-1
    p=[]
    for i in range(0,NbSectors+1,1):
        angle=start+i*dTeta
        v1=[O[0]+cos(angle)-(-v)*sin(angle),O[1]+sin(angle)+(-v)*cos(angle)]
        pt=[O[0]+cos(angle)                ,O[1]+sin(angle)                ]
        v2=[O[0]+cos(angle)-  v *sin(angle),O[1]+sin(angle)+  v *cos(angle)]
        p.append([v1,pt,v2])
    p[ 0][0]=p[ 0][1][:]
    p[-1][2]=p[-1][1][:]

    mat=matprod((rotmat(teta),[[rx,0],[0,ry]],rotmat(-teta)))
    for pts in p:
        applymat(mat, pts[0])
        applymat(mat, pts[1])
        applymat(mat, pts[2])
    return(p)
    
def CubicSuperPath(simplepath):
    csp = []
    subpath = -1
    subpathstart = []
    last = []
    lastctrl = []
    for s in simplepath:
        cmd, params = s        
        if cmd == 'M':
            if last:
                csp[subpath].append([lastctrl[:],last[:],last[:]])
            subpath += 1
            csp.append([])
            subpathstart =  params[:]
            last = params[:]
            lastctrl = params[:]
        elif cmd == 'L':
            csp[subpath].append([lastctrl[:],last[:],last[:]])
            last = params[:]
            lastctrl = params[:]
        elif cmd == 'C':
            csp[subpath].append([lastctrl[:],last[:],params[:2]])
            last = params[-2:]
            lastctrl = params[2:4]
        elif cmd == 'Q':
            q0=last[:]
            q1=params[0:2]
            q2=params[2:4]
            x0=     q0[0]
            x1=1./3*q0[0]+2./3*q1[0]
            x2=           2./3*q1[0]+1./3*q2[0]
            x3=                           q2[0]
            y0=     q0[1]
            y1=1./3*q0[1]+2./3*q1[1]
            y2=           2./3*q1[1]+1./3*q2[1]
            y3=                           q2[1]
            csp[subpath].append([lastctrl[:],[x0,y0],[x1,y1]])
            last = [x3,y3]
            lastctrl = [x2,y2]
        elif cmd == 'A':
            arcp=ArcToPath(last[:],params[:])
            arcp[ 0][0]=lastctrl[:]
            last=arcp[-1][1]
            lastctrl = arcp[-1][0]
            csp[subpath]+=arcp[:-1]
        elif cmd == 'Z':
            csp[subpath].append([lastctrl[:],last[:],last[:]])
            last = subpathstart[:]
            lastctrl = subpathstart[:]
    #append final superpoint
    csp[subpath].append([lastctrl[:],last[:],last[:]])
    return csp    

def unCubicSuperPath(csp):
    a = []
    for subpath in csp:
        if subpath:
            a.append(['M',subpath[0][1][:]])
            for i in range(1,len(subpath)):
                a.append(['C',subpath[i-1][2][:] + subpath[i][0][:] + subpath[i][1][:]])
    return a

def parsePath(d):
    return CubicSuperPath(simplepath.parsePath(d))

def parsePath2(d):
    """Optimized version using numeric encoding"""
    return CubicSuperPath(simplepath.parsePath2(d))

def formatPath(p):
    return simplepath.formatPath(unCubicSuperPath(p))


# vim: expandtab shiftwidth=4 tabstop=8 softtabstop=4 fileencoding=utf-8 textwidth=99



#!/usr/bin/env python
"""
simplestyle.py
Two simple functions for working with inline css
and some color handling on top.

Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

svgcolors={
    'aliceblue':'#f0f8ff',
    'antiquewhite':'#faebd7',
    'aqua':'#00ffff',
    'aquamarine':'#7fffd4',
    'azure':'#f0ffff',
    'beige':'#f5f5dc',
    'bisque':'#ffe4c4',
    'black':'#000000',
    'blanchedalmond':'#ffebcd',
    'blue':'#0000ff',
    'blueviolet':'#8a2be2',
    'brown':'#a52a2a',
    'burlywood':'#deb887',
    'cadetblue':'#5f9ea0',
    'chartreuse':'#7fff00',
    'chocolate':'#d2691e',
    'coral':'#ff7f50',
    'cornflowerblue':'#6495ed',
    'cornsilk':'#fff8dc',
    'crimson':'#dc143c',
    'cyan':'#00ffff',
    'darkblue':'#00008b',
    'darkcyan':'#008b8b',
    'darkgoldenrod':'#b8860b',
    'darkgray':'#a9a9a9',
    'darkgreen':'#006400',
    'darkgrey':'#a9a9a9',
    'darkkhaki':'#bdb76b',
    'darkmagenta':'#8b008b',
    'darkolivegreen':'#556b2f',
    'darkorange':'#ff8c00',
    'darkorchid':'#9932cc',
    'darkred':'#8b0000',
    'darksalmon':'#e9967a',
    'darkseagreen':'#8fbc8f',
    'darkslateblue':'#483d8b',
    'darkslategray':'#2f4f4f',
    'darkslategrey':'#2f4f4f',
    'darkturquoise':'#00ced1',
    'darkviolet':'#9400d3',
    'deeppink':'#ff1493',
    'deepskyblue':'#00bfff',
    'dimgray':'#696969',
    'dimgrey':'#696969',
    'dodgerblue':'#1e90ff',
    'firebrick':'#b22222',
    'floralwhite':'#fffaf0',
    'forestgreen':'#228b22',
    'fuchsia':'#ff00ff',
    'gainsboro':'#dcdcdc',
    'ghostwhite':'#f8f8ff',
    'gold':'#ffd700',
    'goldenrod':'#daa520',
    'gray':'#808080',
    'grey':'#808080',
    'green':'#008000',
    'greenyellow':'#adff2f',
    'honeydew':'#f0fff0',
    'hotpink':'#ff69b4',
    'indianred':'#cd5c5c',
    'indigo':'#4b0082',
    'ivory':'#fffff0',
    'khaki':'#f0e68c',
    'lavender':'#e6e6fa',
    'lavenderblush':'#fff0f5',
    'lawngreen':'#7cfc00',
    'lemonchiffon':'#fffacd',
    'lightblue':'#add8e6',
    'lightcoral':'#f08080',
    'lightcyan':'#e0ffff',
    'lightgoldenrodyellow':'#fafad2',
    'lightgray':'#d3d3d3',
    'lightgreen':'#90ee90',
    'lightgrey':'#d3d3d3',
    'lightpink':'#ffb6c1',
    'lightsalmon':'#ffa07a',
    'lightseagreen':'#20b2aa',
    'lightskyblue':'#87cefa',
    'lightslategray':'#778899',
    'lightslategrey':'#778899',
    'lightsteelblue':'#b0c4de',
    'lightyellow':'#ffffe0',
    'lime':'#00ff00',
    'limegreen':'#32cd32',
    'linen':'#faf0e6',
    'magenta':'#ff00ff',
    'maroon':'#800000',
    'mediumaquamarine':'#66cdaa',
    'mediumblue':'#0000cd',
    'mediumorchid':'#ba55d3',
    'mediumpurple':'#9370db',
    'mediumseagreen':'#3cb371',
    'mediumslateblue':'#7b68ee',
    'mediumspringgreen':'#00fa9a',
    'mediumturquoise':'#48d1cc',
    'mediumvioletred':'#c71585',
    'midnightblue':'#191970',
    'mintcream':'#f5fffa',
    'mistyrose':'#ffe4e1',
    'moccasin':'#ffe4b5',
    'navajowhite':'#ffdead',
    'navy':'#000080',
    'oldlace':'#fdf5e6',
    'olive':'#808000',
    'olivedrab':'#6b8e23',
    'orange':'#ffa500',
    'orangered':'#ff4500',
    'orchid':'#da70d6',
    'palegoldenrod':'#eee8aa',
    'palegreen':'#98fb98',
    'paleturquoise':'#afeeee',
    'palevioletred':'#db7093',
    'papayawhip':'#ffefd5',
    'peachpuff':'#ffdab9',
    'peru':'#cd853f',
    'pink':'#ffc0cb',
    'plum':'#dda0dd',
    'powderblue':'#b0e0e6',
    'purple':'#800080',
    'rebeccapurple':'#663399',
    'red':'#ff0000',
    'rosybrown':'#bc8f8f',
    'royalblue':'#4169e1',
    'saddlebrown':'#8b4513',
    'salmon':'#fa8072',
    'sandybrown':'#f4a460',
    'seagreen':'#2e8b57',
    'seashell':'#fff5ee',
    'sienna':'#a0522d',
    'silver':'#c0c0c0',
    'skyblue':'#87ceeb',
    'slateblue':'#6a5acd',
    'slategray':'#708090',
    'slategrey':'#708090',
    'snow':'#fffafa',
    'springgreen':'#00ff7f',
    'steelblue':'#4682b4',
    'tan':'#d2b48c',
    'teal':'#008080',
    'thistle':'#d8bfd8',
    'tomato':'#ff6347',
    'turquoise':'#40e0d0',
    'violet':'#ee82ee',
    'wheat':'#f5deb3',
    'white':'#ffffff',
    'whitesmoke':'#f5f5f5',
    'yellow':'#ffff00',
    'yellowgreen':'#9acd32'
}

def parseStyle(s):
    """Create a dictionary from the value of an inline style attribute"""
    if s is None:
      return {}
    else:
      return dict([[x.strip() for x in i.split(":")] for i in s.split(";") if len(i.strip())])

def formatStyle(a):
    """Format an inline style attribute from a dictionary"""
    return ";".join([att+":"+str(val) for att,val in a.items()])

def isColor(c):
    """Determine if its a color we can use. If not, leave it unchanged."""
    if c.startswith('#') and (len(c)==4 or len(c)==7):
        return True
    if c.lower() in svgcolors.keys():
        return True
    #might be "none" or some undefined color constant or rgb()
    #however, rgb() shouldnt occur at this point
    return False

def parseColor(c):
    """Creates a rgb int array"""
    tmp = svgcolors.get(c.lower())
    if tmp is not None:
        c = tmp
    elif c.startswith('#') and len(c)==4:
        c='#'+c[1:2]+c[1:2]+c[2:3]+c[2:3]+c[3:]+c[3:]
    elif c.startswith('rgb('):
        # remove the rgb(...) stuff
        tmp = c.strip()[4:-1]
        numbers = [number.strip() for number in tmp.split(',')]
        converted_numbers = []
        if len(numbers) == 3:
            for num in numbers:
                if num.endswith(r'%'):
                    converted_numbers.append(int(float(num[0:-1])*255/100))
                else:
                    converted_numbers.append(int(num))
            return tuple(converted_numbers)
        else:    
            return (0,0,0)
    try:
        r=int(c[1:3],16)
        g=int(c[3:5],16)
        b=int(c[5:],16)
    except:
        # unknown color ...
        # Return a default color. Maybe not the best thing to do but probably
        # better than raising an exception. 
       return(0,0,0)
    return (r,g,b)

def formatColoria(a):
    """int array to #rrggbb"""
    return '#%02x%02x%02x' % (a[0],a[1],a[2])

def formatColorfa(a):
    """float array to #rrggbb"""
    return '#%02x%02x%02x' % (int(round(a[0]*255)),int(round(a[1]*255)),int(round(a[2]*255)))

def formatColor3i(r,g,b):
    """3 ints to #rrggbb"""
    return '#%02x%02x%02x' % (r,g,b)

def formatColor3f(r,g,b):
    """3 floats to #rrggbb"""
    return '#%02x%02x%02x' % (int(round(r*255)),int(round(g*255)),int(round(b*255)))


# vim: expandtab shiftwidth=4 tabstop=8 softtabstop=4 fileencoding=utf-8 textwidth=99



"""
simplepath.py
functions for digesting paths into a simple list structure

Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""
import re
import math

RE_DELIM = re.compile(r'[ \t\r\n,]+')
RE_COMMAND = re.compile(r'[MLHVCSQTAZmlhvcsqtaz]')
RE_PARAMETER = re.compile(r'(([-+]?[0-9]+(\.[0-9]*)?|[-+]?\.[0-9]+)([eE][-+]?[0-9]+)?)')

# The following constants taken from inkex module from Inkscape 1.2:
LEX_REX = re.compile(r"([MLHVCSQTAZmlhvcsqtaz])([^MLHVCSQTAZmlhvcsqtaz]*)")

DIGIT_REX_PART = r"[0-9]"
DIGIT_SEQUENCE_REX_PART = rf"(?:{DIGIT_REX_PART}+)"
INTEGER_CONSTANT_REX_PART = DIGIT_SEQUENCE_REX_PART
SIGN_REX_PART = r"[+-]"
EXPONENT_REX_PART = rf"(?:[eE]{SIGN_REX_PART}?{DIGIT_SEQUENCE_REX_PART})"
FRACTIONAL_CONSTANT_REX_PART =\
    rf"(?:{DIGIT_SEQUENCE_REX_PART}?\.{DIGIT_SEQUENCE_REX_PART}|{DIGIT_SEQUENCE_REX_PART}\.)"
FLOATING_POINT_CONSTANT_REX_PART = rf"(?:{FRACTIONAL_CONSTANT_REX_PART}{EXPONENT_REX_PART}?|{DIGIT_SEQUENCE_REX_PART}{EXPONENT_REX_PART})"
NUMBER_REX = re.compile(
    rf"(?:{SIGN_REX_PART}?{FLOATING_POINT_CONSTANT_REX_PART}|{SIGN_REX_PART}?{INTEGER_CONSTANT_REX_PART})"
)


def lexPath(d):
    """
    returns and iterator that breaks path data
    identifies command and parameter tokens

    Note: This function no longer in use by parsePath (the only
    function that used it), but is left here in case any outside
    software does make use of it.
    """
    offset = 0
    length = len(d)

    while 1:
        m = RE_DELIM.match(d, offset)
        if m:
            offset = m.end()
        if offset >= length:
            break
        m = RE_COMMAND.match(d, offset)
        if m:
            yield [d[offset:m.end()], True]
            offset = m.end()
            continue
        m = RE_PARAMETER.match(d, offset)
        if m:
            yield [d[offset:m.end()], False]
            offset = m.end()
            continue
        #TODO: create new exception
        raise Exception('Invalid path data!')


IMPLICIT_NEXT_CMDS = {
    'M': 'L',
    'm': 'l',
    'L': 'L',
    'l': 'l',
    'H': 'H',
    'h': 'h',
    'V': 'V',
    'v': 'v',
    'C': 'C',
    'c': 'c',
    'S': 'S',
    's': 's',
    'Q': 'Q',
    'q': 'q',
    'T': 'T',
    't': 't',
    'A': 'A',
    'a': 'a',
    'Z': 'L',
    'z': 'l'
    }


def parse_string(path_d):
    """
    Parse path string into commands and parameters.
    Replaces lexPath (with changes made to parsePath).
    Based on parse_string() in inkex from Inkscape 1.2.
    """
    for cmd, numbers in LEX_REX.findall(path_d):
        args = [float(val) for val in NUMBER_REX.findall(numbers)]
        numParams = pathdefs[cmd.upper()][1]
        next_cmd = IMPLICIT_NEXT_CMDS[cmd]
        i = 0
        while i < len(args) or numParams == 0:
            if len(args[i: i + numParams]) != numParams:
                return
            yield cmd, args[i: i + numParams]
            i += numParams
            cmd = next_cmd
            numParams = pathdefs[cmd.upper()][1]


def parse_string2(path_d):
    """
    Parse SVG path string using ASCII command encoding and pre-computed lookups.
    Returns (ascii_cmd, args) tuples for internal processing.
    """
    for cmd_str, numbers in LEX_REX.findall(path_d):
        args = [float(val) for val in NUMBER_REX.findall(numbers)]
        args_len = len(args)  # Cache length

        # Convert to ASCII directly
        cmd_ascii = ord(cmd_str)

        # Direct O(1) lookups using pre-computed tables
        numParams = CMD_PARAM_COUNTS[cmd_ascii]
        next_cmd_ascii = CMD_NEXT_CMDS[cmd_ascii]

        i = 0
        while i < args_len or numParams == 0:
            # Bounds check
            if i + numParams > args_len:
                return

            # Yield ASCII command with args
            yield cmd_ascii, args[i: i + numParams]
            i += numParams
            cmd_ascii = next_cmd_ascii

            # Direct lookup in loop (no list indexing)
            numParams = CMD_PARAM_COUNTS[cmd_ascii]


'''
pathdefs = {commandfamily:
    [
    implicitnext,
    #params,
    [casts,cast,cast],
    [coord type,x,y,0]
    ]}
'''
pathdefs = {
    'M': ['L', 2, [float, float], ['x', 'y']],
    'L': ['L', 2, [float, float], ['x', 'y']],
    'H': ['H', 1, [float], ['x']],
    'V': ['V', 1, [float], ['y']],
    'C': ['C', 6, [float, float, float, float, float, float], ['x', 'y', 'x', 'y', 'x', 'y']],
    'S': ['S', 4, [float, float, float, float], ['x', 'y', 'x', 'y']],
    'Q': ['Q', 4, [float, float, float, float], ['x', 'y', 'x', 'y']],
    'T': ['T', 2, [float, float], ['x', 'y']],
    'A': ['A', 7,
          [float, float, float, int, int, float, float], ['r', 'r', 'a', 0, 's', 'x', 'y']],
    'Z': ['L', 0, [], []]
    }


# ASCII-based command encoding for efficient processing
# Using ASCII values directly: 'A'=65, 'a'=97, etc.
# Uppercase (absolute) commands: 65-90, Lowercase (relative) commands: 97-122
# Enables relative detection: cmd > 96
# Direct conversion available: chr(cmd) gives the letter

# Format: {ascii_code: [next_ascii_code, param_count, [cast_funcs], [coord_types]]}
pathdefs_numeric = {
    77: [76, 2, [float, float], ['x', 'y']],     # 'M' -> 'L'
    109: [108, 2, [float, float], ['x', 'y']],   # 'm' -> 'l'
    76: [76, 2, [float, float], ['x', 'y']],     # 'L' -> 'L'
    108: [108, 2, [float, float], ['x', 'y']],   # 'l' -> 'l'
    72: [72, 1, [float], ['x']],                 # 'H' -> 'H'
    104: [104, 1, [float], ['x']],               # 'h' -> 'h'
    86: [86, 1, [float], ['y']],                 # 'V' -> 'V'
    118: [118, 1, [float], ['y']],               # 'v' -> 'v'
    67: [67, 6, [float, float, float, float, float, float], ['x', 'y', 'x', 'y', 'x', 'y']],  # 'C'
    99: [99, 6, [float, float, float, float, float, float], ['x', 'y', 'x', 'y', 'x', 'y']],  # 'c'
    83: [83, 4, [float, float, float, float], ['x', 'y', 'x', 'y']],     # 'S' -> 'S'
    115: [115, 4, [float, float, float, float], ['x', 'y', 'x', 'y']],   # 's' -> 's'
    81: [81, 4, [float, float, float, float], ['x', 'y', 'x', 'y']],     # 'Q' -> 'Q'
    113: [113, 4, [float, float, float, float], ['x', 'y', 'x', 'y']],   # 'q' -> 'q'
    84: [84, 2, [float, float], ['x', 'y']],     # 'T' -> 'T'
    116: [116, 2, [float, float], ['x', 'y']],   # 't' -> 't'
    65: [65, 7, [float, float, float, int, int, float, float],
         ['r', 'r', 'a', 0, 's', 'x', 'y']],  # 'A'
    97: [97, 7, [float, float, float, int, int, float, float],
         ['r', 'r', 'a', 0, 's', 'x', 'y']],  # 'a'
    90: [76, 0, [], []],                         # 'Z' -> 'L'
    122: [108, 0, [], []]                        # 'z' -> 'l'
}

# Pre-computed lookup tables for efficient processing
CMD_PARAM_COUNTS = {ascii_val: pathdef[1] for ascii_val, pathdef in pathdefs_numeric.items()}
CMD_NEXT_CMDS = {ascii_val: pathdef[0] for ascii_val, pathdef in pathdefs_numeric.items()}

LOWER_CMDS = set(['m', 'l', 'h', 'v', 'c', 's', 'q', 't', 'a', 'z'])


def parsePath(d):
    """
    Parse SVG path and return an array of segments.
    Removes all shorthand notation.
    Converts coordinates to absolute.
    """
    retval = []
    lexer = parse_string(d)

    pen = (0.0, 0.0)
    subPathStart = pen
    lastControl = pen
    lastCommand = ''

    while 1:
        try:
            cmd, args = next(lexer)
        except StopIteration:
            break
        cmd_upper = cmd.upper()
        if not lastCommand and cmd_upper != 'M':
            raise Exception('Invalid path, must begin with moveto.')

        numParams = pathdefs[cmd_upper][1]
        params = []

        for index, value in enumerate(args):
            cast = pathdefs[cmd_upper][2][index]
            param = cast(value)
            if cmd in LOWER_CMDS:
                if pathdefs[cmd_upper][3][index] == 'x':
                    param += pen[0]
                elif pathdefs[cmd_upper][3][index] == 'y':
                    param += pen[1]
            params.append(param)
        outputCommand = cmd_upper  # Since parameters are now absolute

        # Flesh out shortcut notation
        if outputCommand in ('H', 'V'):
            if outputCommand == 'H':
                params.append(pen[1])
            if outputCommand == 'V':
                params.insert(0, pen[0])
            outputCommand = 'L'
        if outputCommand in ('S', 'T'):
            params.insert(0, pen[1]+(pen[1]-lastControl[1]))
            params.insert(0, pen[0]+(pen[0]-lastControl[0]))
            if outputCommand == 'S':
                outputCommand = 'C'
            if outputCommand == 'T':
                outputCommand = 'Q'

        # current values become "last" values
        if outputCommand == 'M':
            subPathStart = tuple(params[0:2])
            pen = subPathStart
        if outputCommand == 'Z':
            pen = subPathStart
        else:
            pen = tuple(params[-2:])

        if outputCommand in ('Q', 'C'):
            lastControl = tuple(params[-4:-2])
        else:
            lastControl = pen
        lastCommand = cmd_upper

        retval.append([outputCommand, params])
    return retval


def parsePath2(d):
    """
    Parse SVG path string using ASCII encoding throughout.
    Uses integer comparisons for efficient command processing.
    Maintains backward compatibility with string commands in output.
    """
    retval = []
    lexer = parse_string2(d)  # Use ASCII parser

    pen = (0.0, 0.0)
    subPathStart = pen
    lastControl = pen
    lastCommand = 0  # Store as ASCII code instead of string

    while 1:
        try:
            cmd_ascii, args = next(lexer)
        except StopIteration:
            break

        # ASCII uppercase conversion using bit manipulation
        cmd_upper_ascii = cmd_ascii & ~32 if cmd_ascii > 96 else cmd_ascii

        if not lastCommand and cmd_upper_ascii != 77:  # 77 = 'M'
            raise Exception('Invalid path, must begin with moveto.')

        # Use ASCII pathdef lookup
        pathdef = pathdefs_numeric[cmd_ascii]
        params = []

        for index, value in enumerate(args):
            cast = pathdef[2][index]
            param = cast(value)
            # Relative check: cmd_ascii > 96 means lowercase
            if cmd_ascii > 96:
                if pathdef[3][index] == 'x':
                    param += pen[0]
                elif pathdef[3][index] == 'y':
                    param += pen[1]
            params.append(param)

        # Convert to string only for output compatibility
        outputCommand = chr(cmd_upper_ascii)

        # Flesh out shortcut notation using ASCII comparisons
        if cmd_upper_ascii == 72 or cmd_upper_ascii == 86:  # 'H'=72, 'V'=86
            if cmd_upper_ascii == 72:  # 'H'
                params.append(pen[1])
            else:  # 'V'
                params.insert(0, pen[0])
            outputCommand = 'L'
            cmd_upper_ascii = 76  # 'L'=76
        elif cmd_upper_ascii == 83 or cmd_upper_ascii == 84:  # 'S'=83, 'T'=84
            params.insert(0, pen[1]+(pen[1]-lastControl[1]))
            params.insert(0, pen[0]+(pen[0]-lastControl[0]))
            if cmd_upper_ascii == 83:  # 'S'
                outputCommand = 'C'
                cmd_upper_ascii = 67  # 'C'=67
            else:  # 'T'
                outputCommand = 'Q'
                cmd_upper_ascii = 81  # 'Q'=81

        # Current values become "last" values - ASCII comparisons
        if cmd_upper_ascii == 77:  # 'M'=77
            subPathStart = tuple(params[0:2])
            pen = subPathStart
        elif cmd_upper_ascii == 90:  # 'Z'=90
            pen = subPathStart
        else:
            pen = tuple(params[-2:])

        if cmd_upper_ascii == 81 or cmd_upper_ascii == 67:  # 'Q'=81, 'C'=67
            lastControl = tuple(params[-4:-2])
        else:
            lastControl = pen
        lastCommand = cmd_upper_ascii

        retval.append([outputCommand, params])
    return retval


def formatPath(a):
    """Format SVG path data from an array"""
    return "".join([cmd + " ".join([str(p) for p in params]) for cmd, params in a])


def translatePath(p, x, y):
    for cmd, params in p:
        defs = pathdefs[cmd]
        for i in range(defs[1]):
            if defs[3][i] == 'x':
                params[i] += x
            elif defs[3][i] == 'y':
                params[i] += y


def scalePath(p, x, y):
    for cmd, params in p:
        defs = pathdefs[cmd]
        for i in range(defs[1]):
            if defs[3][i] == 'x':
                params[i] *= x
            elif defs[3][i] == 'y':
                params[i] *= y
            elif defs[3][i] == 'r':         # radius parameter
                params[i] *= x
            elif defs[3][i] == 's':         # sweep-flag parameter
                if x*y < 0:
                    params[i] = 1 - params[i]
            elif defs[3][i] == 'a':         # x-axis-rotation angle
                if y < 0:
                    params[i] = - params[i]


def rotatePath(p, a, cx=0, cy=0):
    if a == 0:
        return p
    for cmd, params in p:
        defs = pathdefs[cmd]
        for i in range(defs[1]):
            if defs[3][i] == 'x':
                x = params[i] - cx
                y = params[i + 1] - cy
                r = math.sqrt((x**2) + (y**2))
                if r != 0:
                    theta = math.atan2(y, x) + a
                    params[i] = (r * math.cos(theta)) + cx
                    params[i + 1] = (r * math.sin(theta)) + cy


# vim: expandtab shiftwidth=4 tabstop=8 softtabstop=4 fileencoding=utf-8 textwidth=99



#!/usr/bin/env python
"""
    ffgeom_legacy.py
    Copyright (C) 2005 Aaron Cyril Spike, aaron@ekips.org

    This file is part of FretFind 2-D.

    FretFind 2-D is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FretFind 2-D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FretFind 2-D; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
import math
try:
    NaN = float('NaN')
except ValueError:
    PosInf = 1e300000
    NaN = PosInf/PosInf

class Point:
    precision = 5
    def __init__(self, x, y):
        self.__coordinates = {'x' : float(x), 'y' : float(y)}
    def __getitem__(self, key):
        return self.__coordinates[key]
    def __setitem__(self, key, value):
        self.__coordinates[key] = float(value)
    def __repr__(self):
        return '(%s, %s)' % (round(self['x'],self.precision),round(self['y'],self.precision))
    def copy(self):
        return Point(self['x'],self['y'])
    def translate(self, x, y):
        self['x'] += x
        self['y'] += y
    def move(self, x, y):
        self['x'] = float(x)
        self['y'] = float(y)

class Segment:
    def __init__(self, e0, e1):
        self.__endpoints = [e0, e1]
    def __getitem__(self, key):
        return self.__endpoints[key]
    def __setitem__(self, key, value):
        self.__endpoints[key] = value
    def __repr__(self):
        return repr(self.__endpoints)
    def copy(self):
        return Segment(self[0],self[1])
    def translate(self, x, y):
        self[0].translate(x,y)
        self[1].translate(x,y)
    def move(self,e0,e1):
        self[0] = e0
        self[1] = e1
    def delta_x(self):
        return self[1]['x'] - self[0]['x']
    def delta_y(self):
        return self[1]['y'] - self[0]['y']
    #alias functions
    run = delta_x
    rise = delta_y
    def slope(self):
        if self.delta_x() != 0:
            return self.delta_x() / self.delta_y()
        return NaN
    def intercept(self):
        if self.delta_x() != 0:
            return self[1]['y'] - (self[0]['x'] * self.slope())
        return NaN
    def distanceToPoint(self, p):
        s2 = Segment(self[0],p)
        c1 = dot(s2,self)
        if c1 <= 0:
            return Segment(p,self[0]).length()
        c2 = dot(self,self)
        if c2 <= c1:
            return Segment(p,self[1]).length()
        return self.perpDistanceToPoint(p)
    def perpDistanceToPoint(self, p):
        len = self.length()
        if len == 0: return NaN
        return math.fabs(((self[1]['x'] - self[0]['x']) * (self[0]['y'] - p['y'])) - \
            ((self[0]['x'] - p['x']) * (self[1]['y'] - self[0]['y']))) / len
    def angle(self):
        return math.pi * (math.atan2(self.delta_y(), self.delta_x())) / 180
    def length(self):
        return math.sqrt((self.delta_x() ** 2) + (self.delta_y() ** 2))
    def pointAtLength(self, len):
        if self.length() == 0: return Point(NaN, NaN)
        ratio = len / self.length()
        x = self[0]['x'] + (ratio * self.delta_x())
        y = self[0]['y'] + (ratio * self.delta_y())
        return Point(x, y)
    def pointAtRatio(self, ratio):
        if self.length() == 0: return Point(NaN, NaN)
        x = self[0]['x'] + (ratio * self.delta_x())
        y = self[0]['y'] + (ratio * self.delta_y())
        return Point(x, y)
    def createParallel(self, p):
        return Segment(Point(p['x'] + self.delta_x(), p['y'] + self.delta_y()), p)
    def intersect(self, s):
        return intersectSegments(self, s)

def intersectSegments(s1, s2):
    x1 = s1[0]['x']
    x2 = s1[1]['x']
    x3 = s2[0]['x']
    x4 = s2[1]['x']
    
    y1 = s1[0]['y']
    y2 = s1[1]['y']
    y3 = s2[0]['y']
    y4 = s2[1]['y']
    
    denom = ((y4 - y3) * (x2 - x1)) - ((x4 - x3) * (y2 - y1))
    num1 = ((x4 - x3) * (y1 - y3)) - ((y4 - y3) * (x1 - x3))
    num2 = ((x2 - x1) * (y1 - y3)) - ((y2 - y1) * (x1 - x3))

    num = num1

    if denom != 0: 
        x = x1 + ((num / denom) * (x2 - x1))
        y = y1 + ((num / denom) * (y2 - y1))
        return Point(x, y)
    return Point(NaN, NaN)

def dot(s1, s2):
    return s1.delta_x() * s2.delta_x() + s1.delta_y() * s2.delta_y()


# vim: expandtab shiftwidth=4 tabstop=8 softtabstop=4 fileencoding=utf-8 textwidth=99



import functools
import sys

'''
Attempts to
ensure that error conditions are reported (which inkex does not do properly).
When an error is detected (by catching a
'SystemExit'), the program quits with a non-0 exit status.

Assumes that `func` (usually `effect` or `affect`) only raises `SystemExit`
(e.g. by calling `quit()`, `exit()` or `sys.exit()`) on error conditions.
In the inkscape extensions code, this is true as far as I can tell.
For non-inkscape extensions code, this assumption should also be fine, since
there is never a reason to raise a SystemExit in a function unless there's an
error (e.g. `return` could be used instead).

Usage:

if __name__ == '__main__':
    e = AnEffect()
    exit_status.run(e.affect)
'''

def run(func, *args, **kwargs):
    try:
        return func(*args, **kwargs)
    except SystemExit as se:
        if (se.args == (None,) or se.args == ()): # if quit(), exit(), or sys.exit() was called w/o args
            sys.exit("\nIt is probable that an error occurred in the inkscape software.")
        else: # SystemExit was raised with an error code. Just let it happen.
            raise se



import os
import sys

# make sources in "inkex" directories importable from anywhere (e.g., `import inkex`).
# this method of handling dependencies can cause subtle bugs
sys.path.append(os.path.join(os.path.abspath(os.path.dirname(__file__))))




"""
Based on https://github.com/pypa/sampleproject

"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path
from io import open

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ink_extensions',
    version='2.2.0',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/evil-mad/ink_extensions',

    packages=find_packages(exclude=['contrib', 'docs', 'test', 'test.*']),
    install_requires=[
        'lxml'
    ],
    extras_require={
        'dev': [
            'coverage',  # coverage run -m unittest discover && coverage html
        ],
        'test': [
            'mock',
        ],
    },
)



import logging

from ink_extensions import inkex

# alias that makes more semantic sense to how inkex.errormsg is used
emit = inkex.errormsg

class UserMessageHandler(logging.Handler):
    ''' To be used when a message is logged, and the user should receive the message
    (e.g. an error message)'''

    def __init__(self, formatter=None):
        super(UserMessageHandler, self).__init__()

        # if no formatter specified, set the logging formatter to one that passes the message
        # through unchanged (i.e. without adding dates, etc.)
        self.setFormatter(formatter if formatter is not None else logging.Formatter())
        
    def emit(self, record):
        emit(self.format(record))
